# -*- coding: utf-8 -*-
import sys
import os
import time
import routing
import requests
import json
import urllib
from artwork import ArtworkWindow
from xbmc import Player, sleep, log, executebuiltin, translatePath, getCondVisibility
from xbmcaddon import Addon
from xbmcgui import ListItem, getCurrentWindowDialogId
from xbmcplugin import addDirectoryItem, endOfDirectory, setContent, setResolvedUrl


ADDON = Addon()
ADDON_HANDLE = int(sys.argv[1])
PLUGIN = routing.Plugin()
USERDATA_DIR = translatePath('special://userdata/addon_data/plugin.audio.99fm-playlists')


def get_json():
    if not os.path.exists(USERDATA_DIR):
        os.makedirs(USERDATA_DIR)
    playlists_file_path = '{0}/playlists.json'.format(USERDATA_DIR)
    data = None

    if os.path.isfile(playlists_file_path) and time.time() - os.path.getmtime(playlists_file_path) < 86400:
        file = open(playlists_file_path, 'r')
        data = file.read()
        file.close()
    else:
        log('Fetching playlists...')
        req = requests.get('http://nirelbaz.com/kodi/data/playlists.json')
        data = req.text
        file = open(playlists_file_path, 'w+')
        file.write(data)
        file.close()

    return json.loads(data)


@PLUGIN.route('/')
def index():
    """
    Shows channels list
    :return:
    """
    for index, category in enumerate(get_json()):
        try:
            bg_image = urllib.quote(category['bgImage'], safe=':/')
        except:
            bg_image = PLUGIN.addon.getAddonInfo('fanart')
            pass

        # list item:
        li = ListItem(u'{0} - {1}'.format(category['title'].upper(), category['slogan']),
                      iconImage=bg_image,
                      thumbnailImage=bg_image)
        li.setArt({
            'fanart': bg_image
        })
        # directory item:
        addDirectoryItem(
                PLUGIN.handle,
                PLUGIN.url_for(show_playlists, index=index),
                li,
                True
        )
    # end of directory:
    endOfDirectory(PLUGIN.handle)


@PLUGIN.route('/channels/<index>/playlists')
def show_playlists(index):
    """
    Shows playlists in a channel
    :param index: Channel index
    :return:
    """
    for idx, playlist in enumerate(get_json()[int(index)]['playlists']):
        try:
            bg_image = urllib.quote(playlist['bgImage'], safe=':/')
        except:
            bg_image = PLUGIN.addon.getAddonInfo('fanart')
            pass

        # list item:
        li = ListItem(u'{0}'.format(playlist['title'].upper()),
                      iconImage=bg_image,
                      thumbnailImage=bg_image)
        li.setArt({
            'fanart': bg_image
        })
        li.setProperty('IsPlayable', 'true')
        li.setInfo('music', {
            'Title': u'{0} - {1}'.format(playlist['title'].upper(), playlist['slogan']),
        })
        # set the content of the directory
        setContent(ADDON_HANDLE, 'songs')
        # directory item:
        addDirectoryItem(
                PLUGIN.handle,
                PLUGIN.url_for(play_song, channel_index=index, playlist_index=idx),
                li,
                False
        )
    # end of directory:
    endOfDirectory(PLUGIN.handle)


@PLUGIN.route('/channels/<channel_index>/playlists/<playlist_index>')
def play_song(channel_index, playlist_index):
    playlist = get_json()[int(channel_index)]['playlists'][int(playlist_index)]
    try:
        bg_image = urllib.quote(playlist['bgImage'], safe=':/')
    except:
        bg_image = ADDON.getAddonInfo('fanart')
        pass

    # if ADDON.getSetting('doomsday') == 'true':
    #     urllib.urlretrieve(playlist['songs'][0]['url'],
    #                         '{0}/playlist.mp3'.format(USERDATA_DIR))
    #     playlist['songs'][0]['url'] = '{0}/playlist.mp3'.format(USERDATA_DIR)

    url = urllib.quote(playlist['songs'][0]['url'], safe=':/?=')
    try:
        response = requests.head(url, allow_redirects=False)
        if 'location' in response.headers:
            url = response.headers['location']
    except:
        pass

    li = ListItem(playlist['title'],
                  playlist['slogan'], bg_image, bg_image,
                  url)
    li.setArt({'thumb': bg_image, 'fanart': bg_image})
    li.setInfo('music', {'Title': playlist['title'], 'Artist': playlist['slogan']})
    li.setProperty('mimetype', 'audio/mpeg')
    li.setProperty('IsPlayable', 'true')
    li.setInfo('music', {
        'Title': playlist['title'].upper(),
        'Artist': playlist['slogan']
    })

    setResolvedUrl(ADDON_HANDLE, True, listitem=li)
    update_now_playing(playlist, url)


def update_now_playing(playlist, url):
    last_item = None
    label = None

    artwork = ArtworkWindow()
    artwork.show()
    artwork_id = getCurrentWindowDialogId()

    while Player().isPlaying() and Player().getPlayingFile() == url and getCondVisibility('Window.IsVisible({0})'.format(artwork_id)):
        current_time = Player().getTime()

        if current_time < 30:
            last_item = playlist['songs'][0]
        else:
            for item in playlist['songs']:
                if current_time < item['seconds']:
                    break
                last_item = item

        if last_item:
            if label != last_item['title']:
                label = last_item['title']
                bg_image = last_item['icon']

                artwork.overlay.setImage(playlist['bgImage'])
                artwork.cover.setImage(bg_image)
                artwork.song.setLabel('[B]{0}[/B]'.format(last_item['title'].split('-')[0].encode('utf-8')))
                artwork.artist.setLabel(last_item['title'].split('-')[1].encode('utf-8'))

        sleep(5000)

    artwork.close()


def calc_fractional_seconds(time_label):
    time = time_label.split(':')
    return int(time[0]) * 3600 + int(time[1]) * 60 + int(time[2])


if __name__ == '__main__':
    PLUGIN.run()
    setContent(PLUGIN.handle, 'audio')
